<?php
/**
 * @file
 * Functions for compatibility between ls_survey module and Views module
 */

/**
 * Implements hook_views_data().
 */
function ls_survey_views_data() {
  $data = array();
  $data['node_type']['table']['group'] = t('Node type');

  // tables + fields that can be used for SQL Joins
  // Handle type field.
  $data['node_type']['type'] = array(
    'title' => t('Type machine-name'), // The item it appears as on the UI,
    // The help that appears on the UI,
    'help' => t('The type of a node (for example, "blog", "forum", "story", etc).'),
    'field' => array(
      'handler' => 'views_handler_field',
      'click sortable' => TRUE,
    ),
    'sort' => array(
      'handler' => 'views_handler_sort',
    ),
    'filter' => array(
      'handler' => 'views_handler_filter_node_type',
    ),
    'argument' => array(
      'handler' => 'views_handler_argument_node_type',
    ),
    'relationship' => array(
      'label' => t('Answers nodes'),
      'base' => 'node',
      'base field' => 'type',
      // This allows us to not show this relationship if the base is already
      // node so users won't create circular relationships.
      'skip base' => array('node'),
    ),
  );

  // Handle orig_type field.
  $data['node_type']['orig_type'] = array(
    'title' => t('Original type machine name'),
    'help' => t('The original machine-readable name of this node type. This may be different from the current type name'),
    // This allows us to not show this relationship if the base is already
    // node_revisions so users won't create circular relationships.
    'field' => array(
      'handler' => 'views_handler_field',
      'click sortable' => TRUE,
    ),
    'sort' => array(
      'handler' => 'views_handler_sort',
    ),
    'filter' => array(
      'handler' => 'views_handler_filter_string',
    ),
    'argument' => array(
      'handler' => 'views_handler_argument_string',
    ),
    'relationship' => array(
      'label' => t('Answers content type'),
      'base' => 'ls_survey',
      'base field' => 'ls_orig_type',
      'skip base' => array('node'),
    ),
  );

  $data['node_type']['table']['join'] = array(
    'node' => array(
      'left_field' => 'type',
      'field' => 'type',
    ),
    'ls_answ' => array(
      'left_table' => 'node',
      'left_field' => 'type',
      'field' => 'type',
    ),
  );

  $data['ls_survey']['table']['group'] = t('LS surveys');
  // tables + fields that can be used for SQL Joins
  $data['ls_survey']['table']['join'] = array(
    'node_revisions' => array(
      'left_field' => 'vid',
      'field' => 'vid',
    ),
    'node' => array(
      'left_field' => 'vid',
      'field' => 'vid',
    ),
    'node_type' => array(
      'left_field' => 'orig_type',
      'field' => 'ls_orig_type',
    ),
  );

  module_load_include('inc', 'ls_survey', 'ls_survey');
  // ls_sid
  $data['ls_survey']['ls_sid'] = array(
    'title' => ls_survey_display_sync_headers('ls_sid'),
    'help' => t('Default survey ID'),
    'field' => array(
      'handler' => 'ls_survey_views_handlers_field_ls_fields',
      'click sortable' => TRUE,
    ),
    'filter' => array(
      'handler' => 'views_handler_filter_numeric',
    ),
    'sort' => array(
      'handler' => 'views_handler_sort',
    ),
  );

  // ls_lang
  $data['ls_survey']['ls_lang'] = array(
    'title' => ls_survey_display_sync_headers('ls_lang'),
    'help' => t('Default survey language'),
    'field' => array(
      'handler' => 'ls_survey_views_handlers_field_ls_fields',
      'click sortable' => TRUE,
    ),
    'sort' => array(
      'handler' => 'views_handler_sort',
    ),
    'filter' => array(
      'handler' => 'views_handler_filter_string',
    ),
    'argument' => array(
      'handler' => 'views_handler_argument_string',
    ),
  );

  // Handle ls_orig_type.
  $data['ls_survey']['ls_orig_type'] = array(
    'title' => ls_survey_display_sync_headers('ls_orig_type'),
    'help' => t('Type of the associated answers content type'),
    'field' => array(
      'handler' => 'ls_survey_views_handlers_field_ls_fields',
      'click sortable' => TRUE,
    ),
    'sort' => array(
      'handler' => 'views_handler_sort',
    ),
    'filter' => array(
      'handler' => 'views_handler_filter_string',
    ),
    'argument' => array(
      'handler' => 'views_handler_argument_string',
    ),
    'relationship' => array(
      'label' => t('Associated answers content type'),
      'base' => 'node_type',
      'base field' => 'orig_type',
    ),
  );

  $data['ls_survey']['table']['base'] = array(
    'field' => 'vid',
    'title' => t('LimeSurvey surveys'),
    'help' => t("LimeSurvey surveys nodes"),
    'weight' => 1,
  );

  // When using the new 'LimeSurvey' type you need to use relationships to
  // access fields in other tables.

  // Relationship to the 'Node revision' table

  $data['ls_survey']['vid'] = array(
    'title' => t('Survey revisions'),
    'help' => t('LimeSurvey survey node revision'),
    'relationship' => array(
      'label' => t('Node revision'),
      'base' => 'node_revisions',
      'base field' => 'vid',
      // This allows us to not show this relationship if the base is already
      // node_revisions so users won't create circular relationships.
      'skip base' => array('node'),
    ),
  );

  // Relationship to the 'Node' table
  $data['ls_survey']['nid'] = array(
    'title' => t('Survey node'),
    'help' => t('LimeSurvey survey node'),
    'relationship' => array(
      'label' => t('Survey node'),
      'base' => 'node',
      'base field' => 'nid',
      // This allows us to not show this relationship if the base is already
      // node so users won't create circular relationships.
      'skip base' => array('node', 'node_revisions'),
    ),
  );

  return $data;
}

/**
 * Implements hook_views_handlers().
 */
function ls_survey_views_handlers() {
  return array(
    'info' => array(
      'path' => drupal_get_path('module', 'ls_survey') . '/views',
    ),
    'handlers' => array(
      'ls_survey_views_handlers_field_ls_fields' => array(
        'parent' => 'views_handler_field',
      ),
    ),
  );
}

/**
 * Implements hook_views_default_views().
 */
function ls_survey_views_default_views() {
  $view = new view;
  $view->name = 'surveys_and_associated_answers';
  $view->description = 'This is a basic start in order to create views lists. It query all surveys with their associated answers.';
  $view->tag = '';
  $view->base_table = 'node';
  $view->core = 6;
  $view->api_version = '2';
  $view->disabled = FALSE; /* Edit this to true to make a default view disabled initially */
  $handler = $view->new_display('default', 'Defaults', 'default');
  $handler->override_option('relationships', array(
    'ls_orig_type' => array(
      'label' => 'Associated answers content type',
      'required' => 0,
      'id' => 'ls_orig_type',
      'table' => 'ls_survey',
      'field' => 'ls_orig_type',
      'relationship' => 'none',
    ),
    'type' => array(
      'label' => 'Answers nodes',
      'required' => 0,
      'id' => 'type',
      'table' => 'node_type',
      'field' => 'type',
      'relationship' => 'ls_orig_type',
    ),
  ));
  $handler->override_option('fields', array(
    'title' => array(
      'label' => 'Title',
      'alter' => array(
        'alter_text' => 0,
        'text' => '',
        'make_link' => 0,
        'path' => '',
        'absolute' => 0,
        'link_class' => '',
        'alt' => '',
        'rel' => '',
        'prefix' => '',
        'suffix' => '',
        'target' => '',
        'help' => '',
        'trim' => 0,
        'max_length' => '',
        'word_boundary' => 1,
        'ellipsis' => 1,
        'html' => 0,
        'strip_tags' => 0,
      ),
      'empty' => '',
      'hide_empty' => 0,
      'empty_zero' => 0,
      'hide_alter_empty' => 1,
      'link_to_node' => 1,
      'exclude' => 0,
      'id' => 'title',
      'table' => 'node',
      'field' => 'title',
      'relationship' => 'none',
    ),
    'ls_sid' => array(
      'label' => 'Default survey ID',
      'alter' => array(
        'alter_text' => 0,
        'text' => '',
        'make_link' => 0,
        'path' => '',
        'absolute' => 0,
        'link_class' => '',
        'alt' => '',
        'rel' => '',
        'prefix' => '',
        'suffix' => '',
        'target' => '',
        'help' => '',
        'trim' => 0,
        'max_length' => '',
        'word_boundary' => 1,
        'ellipsis' => 1,
        'html' => 0,
        'strip_tags' => 0,
      ),
      'empty' => '',
      'hide_empty' => 0,
      'empty_zero' => 0,
      'hide_alter_empty' => 1,
      'exclude' => 0,
      'id' => 'ls_sid',
      'table' => 'ls_survey',
      'field' => 'ls_sid',
      'relationship' => 'none',
    ),
    'ls_lang' => array(
      'label' => 'Default survey language',
      'alter' => array(
        'alter_text' => 0,
        'text' => '',
        'make_link' => 0,
        'path' => '',
        'absolute' => 0,
        'link_class' => '',
        'alt' => '',
        'rel' => '',
        'prefix' => '',
        'suffix' => '',
        'target' => '',
        'help' => '',
        'trim' => 0,
        'max_length' => '',
        'word_boundary' => 1,
        'ellipsis' => 1,
        'html' => 0,
        'strip_tags' => 0,
      ),
      'empty' => '',
      'hide_empty' => 0,
      'empty_zero' => 0,
      'hide_alter_empty' => 1,
      'exclude' => 0,
      'id' => 'ls_lang',
      'table' => 'ls_survey',
      'field' => 'ls_lang',
      'relationship' => 'none',
    ),
    'type' => array(
      'label' => 'Type machine-name',
      'alter' => array(
        'alter_text' => 0,
        'text' => '',
        'make_link' => 0,
        'path' => '',
        'absolute' => 0,
        'link_class' => '',
        'alt' => '',
        'rel' => '',
        'prefix' => '',
        'suffix' => '',
        'target' => '',
        'help' => '',
        'trim' => 0,
        'max_length' => '',
        'word_boundary' => 1,
        'ellipsis' => 1,
        'html' => 0,
        'strip_tags' => 0,
      ),
      'empty' => '',
      'hide_empty' => 0,
      'empty_zero' => 0,
      'hide_alter_empty' => 1,
      'exclude' => 0,
      'id' => 'type',
      'table' => 'node_type',
      'field' => 'type',
      'relationship' => 'none',
    ),
    'orig_type' => array(
      'label' => 'Answ Original type machine name',
      'alter' => array(
        'alter_text' => 0,
        'text' => '',
        'make_link' => 0,
        'path' => '',
        'absolute' => 0,
        'link_class' => '',
        'alt' => '',
        'rel' => '',
        'prefix' => '',
        'suffix' => '',
        'target' => '',
        'help' => '',
        'trim' => 0,
        'max_length' => '',
        'word_boundary' => 1,
        'ellipsis' => 1,
        'html' => 0,
        'strip_tags' => 0,
      ),
      'empty' => '',
      'hide_empty' => 0,
      'empty_zero' => 0,
      'hide_alter_empty' => 1,
      'exclude' => 0,
      'id' => 'orig_type',
      'table' => 'node_type',
      'field' => 'orig_type',
      'relationship' => 'ls_orig_type',
    ),
    'ls_orig_type' => array(
      'label' => 'Answers content type',
      'alter' => array(
        'alter_text' => 0,
        'text' => '',
        'make_link' => 0,
        'path' => '',
        'absolute' => 0,
        'link_class' => '',
        'alt' => '',
        'rel' => '',
        'prefix' => '',
        'suffix' => '',
        'target' => '',
        'help' => '',
        'trim' => 0,
        'max_length' => '',
        'word_boundary' => 1,
        'ellipsis' => 1,
        'html' => 0,
        'strip_tags' => 0,
      ),
      'empty' => '',
      'hide_empty' => 0,
      'empty_zero' => 0,
      'hide_alter_empty' => 1,
      'exclude' => 0,
      'id' => 'ls_orig_type',
      'table' => 'ls_survey',
      'field' => 'ls_orig_type',
      'relationship' => 'none',
    ),
    'nid' => array(
      'label' => 'Nid',
      'alter' => array(
        'alter_text' => 0,
        'text' => '',
        'make_link' => 0,
        'path' => '',
        'absolute' => 0,
        'link_class' => '',
        'alt' => '',
        'rel' => '',
        'prefix' => '',
        'suffix' => '',
        'target' => '',
        'help' => '',
        'trim' => 0,
        'max_length' => '',
        'word_boundary' => 1,
        'ellipsis' => 1,
        'html' => 0,
        'strip_tags' => 0,
      ),
      'empty' => '',
      'hide_empty' => 0,
      'empty_zero' => 0,
      'hide_alter_empty' => 1,
      'link_to_node' => 0,
      'exclude' => 0,
      'id' => 'nid',
      'table' => 'node',
      'field' => 'nid',
      'relationship' => 'type',
    ),
    'title_1' => array(
      'label' => 'Title',
      'alter' => array(
        'alter_text' => 0,
        'text' => '',
        'make_link' => 0,
        'path' => '',
        'absolute' => 0,
        'link_class' => '',
        'alt' => '',
        'rel' => '',
        'prefix' => '',
        'suffix' => '',
        'target' => '',
        'help' => '',
        'trim' => 0,
        'max_length' => '',
        'word_boundary' => 1,
        'ellipsis' => 1,
        'html' => 0,
        'strip_tags' => 0,
      ),
      'empty' => '',
      'hide_empty' => 0,
      'empty_zero' => 0,
      'hide_alter_empty' => 1,
      'link_to_node' => 1,
      'exclude' => 0,
      'id' => 'title_1',
      'table' => 'node',
      'field' => 'title',
      'relationship' => 'type',
    ),
    'ls_status' => array(
      'label' => 'Answer status',
      'alter' => array(
        'alter_text' => 0,
        'text' => '',
        'make_link' => 0,
        'path' => '',
        'absolute' => 0,
        'link_class' => '',
        'alt' => '',
        'rel' => '',
        'prefix' => '',
        'suffix' => '',
        'target' => '',
        'help' => '',
        'trim' => 0,
        'max_length' => '',
        'word_boundary' => 1,
        'ellipsis' => 1,
        'html' => 0,
        'strip_tags' => 0,
      ),
      'empty' => '',
      'hide_empty' => 0,
      'empty_zero' => 0,
      'hide_alter_empty' => 1,
      'exclude' => 0,
      'id' => 'ls_status',
      'table' => 'ls_answ',
      'field' => 'ls_status',
      'relationship' => 'type',
    ),
    'ls_sync' => array(
      'label' => 'Synchronization status',
      'alter' => array(
        'alter_text' => 0,
        'text' => '',
        'make_link' => 0,
        'path' => '',
        'absolute' => 0,
        'link_class' => '',
        'alt' => '',
        'rel' => '',
        'prefix' => '',
        'suffix' => '',
        'target' => '',
        'help' => '',
        'trim' => 0,
        'max_length' => '',
        'word_boundary' => 1,
        'ellipsis' => 1,
        'html' => 0,
        'strip_tags' => 0,
      ),
      'empty' => '',
      'hide_empty' => 0,
      'empty_zero' => 0,
      'hide_alter_empty' => 1,
      'exclude' => 0,
      'id' => 'ls_sync',
      'table' => 'ls_answ',
      'field' => 'ls_sync',
      'relationship' => 'type',
    ),
    'ls_last_import' => array(
      'label' => 'Last import',
      'alter' => array(
        'alter_text' => 0,
        'text' => '',
        'make_link' => 0,
        'path' => '',
        'absolute' => 0,
        'link_class' => '',
        'alt' => '',
        'rel' => '',
        'prefix' => '',
        'suffix' => '',
        'target' => '',
        'help' => '',
        'trim' => 0,
        'max_length' => '',
        'word_boundary' => 1,
        'ellipsis' => 1,
        'html' => 0,
        'strip_tags' => 0,
      ),
      'empty' => '',
      'hide_empty' => 0,
      'empty_zero' => 0,
      'hide_alter_empty' => 1,
      'date_format' => 'small',
      'custom_date_format' => '',
      'exclude' => 0,
      'id' => 'ls_last_import',
      'table' => 'ls_answ',
      'field' => 'ls_last_import',
      'relationship' => 'type',
    ),
    'ls_aid' => array(
      'label' => 'Answer ID',
      'alter' => array(
        'alter_text' => 0,
        'text' => '',
        'make_link' => 0,
        'path' => '',
        'absolute' => 0,
        'link_class' => '',
        'alt' => '',
        'rel' => '',
        'prefix' => '',
        'suffix' => '',
        'target' => '',
        'help' => '',
        'trim' => 0,
        'max_length' => '',
        'word_boundary' => 1,
        'ellipsis' => 1,
        'html' => 0,
        'strip_tags' => 0,
      ),
      'empty' => '',
      'hide_empty' => 0,
      'empty_zero' => 0,
      'hide_alter_empty' => 1,
      'exclude' => 0,
      'id' => 'ls_aid',
      'table' => 'ls_answ',
      'field' => 'ls_aid',
      'relationship' => 'type',
    ),
    'ls_sid_1' => array(
      'label' => 'Survey ID',
      'alter' => array(
        'alter_text' => 0,
        'text' => '',
        'make_link' => 0,
        'path' => '',
        'absolute' => 0,
        'link_class' => '',
        'alt' => '',
        'rel' => '',
        'prefix' => '',
        'suffix' => '',
        'target' => '',
        'help' => '',
        'trim' => 0,
        'max_length' => '',
        'word_boundary' => 1,
        'ellipsis' => 1,
        'html' => 0,
        'strip_tags' => 0,
      ),
      'empty' => '',
      'hide_empty' => 0,
      'empty_zero' => 0,
      'hide_alter_empty' => 1,
      'exclude' => 0,
      'id' => 'ls_sid_1',
      'table' => 'ls_answ',
      'field' => 'ls_sid',
      'relationship' => 'type',
    ),
    'ls_lang_1' => array(
      'label' => 'Survey language',
      'alter' => array(
        'alter_text' => 0,
        'text' => '',
        'make_link' => 0,
        'path' => '',
        'absolute' => 0,
        'link_class' => '',
        'alt' => '',
        'rel' => '',
        'prefix' => '',
        'suffix' => '',
        'target' => '',
        'help' => '',
        'trim' => 0,
        'max_length' => '',
        'word_boundary' => 1,
        'ellipsis' => 1,
        'html' => 0,
        'strip_tags' => 0,
      ),
      'empty' => '',
      'hide_empty' => 0,
      'empty_zero' => 0,
      'hide_alter_empty' => 1,
      'exclude' => 0,
      'id' => 'ls_lang_1',
      'table' => 'ls_answ',
      'field' => 'ls_lang',
      'relationship' => 'type',
    ),
  ));
  $handler->override_option('filters', array(
    'type' => array(
      'operator' => 'in',
      'value' => array(
        'survey' => 'survey',
      ),
      'group' => '0',
      'exposed' => FALSE,
      'expose' => array(
        'operator' => FALSE,
        'label' => '',
      ),
      'id' => 'type',
      'table' => 'node',
      'field' => 'type',
      'relationship' => 'none',
    ),
  ));
  $handler->override_option('access', array(
    'type' => 'none',
  ));
  $handler->override_option('cache', array(
    'type' => 'none',
  ));
  $handler->override_option('style_plugin', 'table');
  $handler->override_option('style_options', array(
    'grouping' => '',
    'override' => 1,
    'sticky' => 0,
    'order' => 'asc',
    'summary' => '',
    'columns' => array(
      'title' => 'title',
      'ls_sid' => 'ls_sid',
      'ls_lang' => 'ls_lang',
    ),
    'info' => array(
      'title' => array(
        'sortable' => 0,
        'separator' => '',
      ),
      'ls_sid' => array(
        'sortable' => 0,
        'separator' => '',
      ),
      'ls_lang' => array(
        'sortable' => 0,
        'separator' => '',
      ),
    ),
    'default' => '-1',
  ));

  // Add view to list of views to provide.
  $views[$view->name] = $view;

  // At the end, return array of default views.
  return $views;
}
